package de.donmanfred;

import java.util.Iterator;
import java.util.Map;

import com.firebase.ui.auth.AuthUI;
import com.firebase.ui.auth.AuthUI.SignInIntentBuilder;
import com.google.android.gms.tasks.OnCompleteListener;
import com.google.android.gms.tasks.OnFailureListener;
import com.google.android.gms.tasks.Task;
import com.google.firebase.auth.AuthResult;
import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.auth.FirebaseAuth.AuthStateListener;
import com.google.firebase.auth.FirebaseAuth.IdTokenListener;
import com.google.firebase.auth.FirebaseUser;
import com.google.firebase.auth.UserProfileChangeRequest;

import android.content.Intent;
import anywheresoftware.b4a.AbsObjectWrapper;
import anywheresoftware.b4a.BA;
import anywheresoftware.b4a.BA.ActivityObject;
import anywheresoftware.b4a.BA.Events;
import anywheresoftware.b4a.BA.Hide;
import anywheresoftware.b4a.BA.ShortName;
import anywheresoftware.b4a.IOnActivityResult;


@BA.Version(0.45F)
@BA.ShortName("FirebaseUI")
@BA.Author("DonManfred (wrapper)")
@ActivityObject()
@Events(values={
	"TokenAvailable(user As FirebaseUser, success As Boolean, token As String)",
	"AccessTokenAvailable(success As Boolean, token As String)",
	"SignedIn(user As firebaseUser)",
	"IdTokenChanged(user As firebaseUser)",
	"SignedOut()",
	"UpdateEmail(success As Boolean, email As String)",
	"UpdatePassword(success As Boolean, password As String)",
	"UpdateProfile(success As Boolean, request As Object)",
})
//@DependsOn(values={
//	"firebase-ui.aar",
//	"firebase-ui-auth.aar",
//	"firebase-ui-common.aar",
//	"firebase-ui-database.aar",
//	"firebase-ui-firestore.aar",
//	"firebase-ui-storage.aar"
//})

public class FirebaseUIWrapper {
	private BA ba;
	private String eventName;
	private FirebaseAuth auth;
	private AuthUI aui;
	private AuthStateListener listener = null;
	private IOnActivityResult ion;
	private static final String EXTRA_IDP_RESPONSE = "extra_idp_response";
  private static final String EXTRA_SIGNED_IN_CONFIG = "extra_signed_in_config";

	public FirebaseUIWrapper() {
	}
	public void Initialize(final BA ba, String EventName, String defaultClient) {
		eventName = EventName.toLowerCase(BA.cul);
		this.ba = ba;
		//if (listener == null){
		//	AuthStateListener aslistener = CreateListener();
		//	listener = aslistener;
		//}
		this.auth = FirebaseAuth.getInstance();
		//AddListener(listener);
		//BA.Log("auth: "+auth.getLanguageCode());
		this.aui = AuthUI.getInstance();
		//BA.Log("FBUI.Initialize Calling AddListener");
		if (auth.getCurrentUser() != null) {
      FirebaseUser user = auth.getCurrentUser();
      if(user != null) {
         ba.raiseEventFromDifferentThread(FirebaseUIWrapper.this, (Object)null, 0, FirebaseUIWrapper.this.eventName + "_signedin", false, new Object[]{AbsObjectWrapper.ConvertToWrapper(new FirebaseUIWrapper.FirebaseUserWrapper(), user)});
      }
		} else {
	    // not signed in
		}
		//return listener;
	}
	public IdTokenListener CreateIdTokenListener(){
		IdTokenListener listener = new IdTokenListener(){

			@Override
			public void onIdTokenChanged(FirebaseAuth auth) {
				// TODO Auto-generated method stub
			 //String refreshedToken = FirebaseInstanceId.getInstance().getToken();
			 //   Log.d(TAG, "Refreshed token: " + refreshedToken);
			 Runnable r = new Runnable() {

         @Override
         public void run() {
        	 FirebaseUser user = FirebaseAuth.getInstance().getCurrentUser();
        	 ba.raiseEventFromDifferentThread(FirebaseUIWrapper.this, null, 0, eventName + "_tokenavailable", true, new Object[] {AbsObjectWrapper.ConvertToWrapper(new FirebaseUIWrapper.FirebaseUserWrapper(), user),true,""});
         }
         
			 };
			 BA.submitRunnable(r, this, 0);
			 ba.raiseEventFromDifferentThread(FirebaseUIWrapper.this, (Object)null, 0, FirebaseUIWrapper.this.eventName + "_idtokenchanged", false, new Object[]{AbsObjectWrapper.ConvertToWrapper(new FirebaseUIWrapper.FirebaseUserWrapper(), auth.getCurrentUser())});
			}
		};
		return listener;
	}
	public void AddIdTokenListener(IdTokenListener listener){
    this.auth.addIdTokenListener(listener);
	}
	public void RemoveIdTokenListener(IdTokenListener listener){
    this.auth.removeIdTokenListener(listener);
	}

	public AuthStateListener CreateListener(){
		//BA.Log("Creating new Listener");
		AuthStateListener	aslistener = new AuthStateListener() {
			@Override
			public void onAuthStateChanged(FirebaseAuth firebaseAuth) {
				//BA.Log("onAuthStateChanged()");
				FirebaseUser user = null;
				if (firebaseAuth != null){
					//BA.Log("GetCurrentUser");
					user = firebaseAuth.getCurrentUser();
				}
				if(user != null) {
					//BA.Log("User != null");
	      		
					ba.raiseEventFromDifferentThread(FirebaseUIWrapper.this, (Object)null, 0, FirebaseUIWrapper.this.eventName + "_signedin", false, new Object[]{AbsObjectWrapper.ConvertToWrapper(new FirebaseUIWrapper.FirebaseUserWrapper(), user)});
				} else {
					//BA.Log("User == null");
					ba.raiseEventFromDifferentThread(FirebaseUIWrapper.this, (Object)null, 0, FirebaseUIWrapper.this.eventName + "_signedin", false, new Object[]{null});
				}
			}
		};
		return aslistener;
	}
	public void AddListener(AuthStateListener listener){
		if (listener != null){
			//BA.Log("FBUI.AddListener Calling RemoveAuthStateListener");
			//this.auth.removeAuthStateListener(listener);
		} else {
			//BA.Log("FBUI.AddListener Calling CreateListener");
			//CreateListener();
		}
		//BA.Log("FBUI.AddListener Calling addAuthStateListener");
		this.auth.addAuthStateListener(listener);
	}
	public void RemoveListener(AuthStateListener listener){
		//BA.Log("FBUI.RemoveListener");
		if (listener != null){
			//BA.Log("FBUI.RemoveListener Calling RemoveAuthStateListener");
			this.auth.removeAuthStateListener(listener);
		}
	}

	//	public void AddAuthStateListener(AuthStateListener listener){
	//    this.auth.addAuthStateListener(listener);
	//	}
	//	public void RemoveAuthStateListener(AuthStateListener listener){
	//    this.auth.removeAuthStateListener(listener);
	//	}
	
	public SignInIntentBuilder createSignInIntentBuilder(){
		return this.aui.createSignInIntentBuilder();
	}
//	public void GetAccessToken(final boolean arg0){
//		String result = FirebaseAuth.getInstance().getAccessToken(arg0).addOnCompleteListener(new OnCompleteListener<GetTokenResult>(){
//			@Override
//			public void onComplete(Task<GetTokenResult> result) {
//				// TODO Auto-generated method stub
//				if (result.isSuccessful() == true){
//					String accessToken = result.getResult().getToken();
//					ba.raiseEventFromDifferentThread(FirebaseUIWrapper.this, null, 0, eventName + "_accesstokenavailable", true, new Object[] {Boolean.TRUE,accessToken});
//				} else {
//					String accessToken = "";
//					ba.raiseEventFromDifferentThread(FirebaseUIWrapper.this, null, 0, eventName + "_accesstokenavailable", true, new Object[] {Boolean.FALSE,accessToken});
//				}
//			}
//		});
//
//	}

	
	public void SendSigninIntent(Intent signin){
	  ion = new IOnActivityResult() {
	  	@Override
	  	public void ResultArrived(int resultCode, Intent data) {
	  		//BA.Log("ResultArrived("+resultCode+")");
	  		//BA.Log("ResultArrived()");
	  		//ba.Log(data.getExtras().keySet().toString());
	  		Iterator it = data.getExtras().keySet().iterator();
	      while (it.hasNext()) {
	      	Map.Entry pair = (Map.Entry)it.next();
	      	//BA.Log(pair.getKey() + " = " + pair.getValue());
	      	it.remove(); // avoids a ConcurrentModificationException
	      }
//	  		if (auth.getCurrentUser() != null) {
//	        FirebaseUser user = auth.getCurrentUser();
//	        if(user != null) {
//	           ba.raiseEventFromDifferentThread(FirebaseUIWrapper.this, (Object)null, 0, FirebaseUIWrapper.this.eventName + "_signedin", false, new Object[]{AbsObjectWrapper.ConvertToWrapper(new FirebaseUIWrapper.FirebaseUserWrapper(), user)});
//	        }
//	  		} else {
//	  	    // not signed in
//	  		}

	      

	  	}	
	  };		
	  ba.startActivityForResult(ion, signin);
	}
	
	
	@SuppressWarnings({ "unchecked", "rawtypes" })
	public void signOut() {
		auth.signOut();
		//aui.signOut(ba.context);
	}
	
	
	@SuppressWarnings("unchecked")
	//@Hide


	public void signInWithCustomToken(final String token) {
		auth.signInWithCustomToken(token).addOnCompleteListener(new OnCompleteListener<AuthResult>(){

			@Override
			public void onComplete(Task<AuthResult> tsk) {
				// TODO Auto-generated method stub
				if (tsk.isSuccessful() == true){
	        FirebaseUser user = tsk.getResult().getUser();
	        if(user != null) {
	           ba.raiseEventFromDifferentThread(FirebaseUIWrapper.this, (Object)null, 0, FirebaseUIWrapper.this.eventName + "_signedin", false, new Object[]{AbsObjectWrapper.ConvertToWrapper(new FirebaseUIWrapper.FirebaseUserWrapper(), user)});
	        }
				} else {
					ba.raiseEventFromDifferentThread(FirebaseUIWrapper.this, null, 0, FirebaseUIWrapper.this.eventName + "_signedin", true, new Object[] {null});
				}
				
			}
     }).addOnFailureListener(new OnFailureListener(){

			@Override
			public void onFailure(Exception arg0) {
				// TODO Auto-generated method stub
				ba.raiseEventFromDifferentThread(FirebaseUIWrapper.this, null, 0, FirebaseUIWrapper.this.eventName + "_updateemail", true, new Object[] {Boolean.valueOf(false), ""});
				
			}
		});
	}
	public void updateEmail(final String newEmail) {
		auth.getCurrentUser().updateEmail(newEmail).addOnCompleteListener(new OnCompleteListener<Void>(){

			@Override
			public void onComplete(Task<Void> tsk) {
				// TODO Auto-generated method stub
				if (tsk.isSuccessful() == true){
					ba.raiseEventFromDifferentThread(FirebaseUIWrapper.this, null, 0, FirebaseUIWrapper.this.eventName + "_updateemail", true, new Object[] {Boolean.valueOf(true), newEmail});
				} else {
					ba.raiseEventFromDifferentThread(FirebaseUIWrapper.this, null, 0, FirebaseUIWrapper.this.eventName + "_updateemail", true, new Object[] {Boolean.valueOf(false), newEmail});
				}
				
			}
     }).addOnFailureListener(new OnFailureListener(){

			@Override
			public void onFailure(Exception arg0) {
				// TODO Auto-generated method stub
				ba.raiseEventFromDifferentThread(FirebaseUIWrapper.this, null, 0, FirebaseUIWrapper.this.eventName + "_updateemail", true, new Object[] {Boolean.valueOf(false), ""});
				
			}
		});
	}
	@SuppressWarnings({ "unchecked", "rawtypes" })
	//@Hide
	public void updatePassword(final String newPassword) {
		auth.getCurrentUser().updatePassword(newPassword).addOnCompleteListener(new OnCompleteListener(){

			@Override
			public void onComplete(Task tsk) {
				// TODO Auto-generated method stub
				if (tsk.isSuccessful() == true){
					ba.raiseEventFromDifferentThread(FirebaseUIWrapper.this, null, 0, FirebaseUIWrapper.this.eventName + "_updatepassword", true, new Object[] {Boolean.valueOf(true), newPassword});
				} else {
					ba.raiseEventFromDifferentThread(FirebaseUIWrapper.this, null, 0, FirebaseUIWrapper.this.eventName + "_updatepassword", true, new Object[] {Boolean.valueOf(false), newPassword});
				}
			}
     }).addOnFailureListener(new OnFailureListener(){

			@Override
			public void onFailure(Exception arg0) {
				// TODO Auto-generated method stub
				ba.raiseEventFromDifferentThread(FirebaseUIWrapper.this, null, 0, FirebaseUIWrapper.this.eventName + "_updatepassword", true, new Object[] {Boolean.valueOf(false), ""});
				
			}
		});
	}
	@SuppressWarnings("unchecked")
	@Hide
	public void updateProfile(final UserProfileChangeRequest request) {
//		auth.getCurrentUser().updateProfile(request).addOnCompleteListener(new OnCompleteListener(){
//
//			@Override
//			public void onComplete(Task tsk) {
//				// TODO Auto-generated method stub
//				if (tsk.isSuccessful() == true){
//					ba.raiseEventFromDifferentThread(FirebaseUIWrapper.this, null, 0, FirebaseUIWrapper.this.eventName + "_updateprofile", true, new Object[] {Boolean.valueOf(true), request});
//				} else {
//					ba.raiseEventFromDifferentThread(FirebaseUIWrapper.this, null, 0, FirebaseUIWrapper.this.eventName + "_updateprofile", true, new Object[] {Boolean.valueOf(false), request});
//				}
//			}
//     }).addOnFailureListener(new OnFailureListener(){
//
//			@Override
//			public void onFailure(Exception arg0) {
//				// TODO Auto-generated method stub
//				ba.raiseEventFromDifferentThread(FirebaseUIWrapper.this, null, 0, FirebaseUIWrapper.this.eventName + "_updateprofile", true, new Object[] {Boolean.valueOf(false), ""});
//				
//			}
//		});
	}

	
  public FirebaseUIWrapper.FirebaseUserWrapper getCurrentUser() {
  	//this.auth.getCurrentUser()
		if (auth.getCurrentUser() != null) {
			return (FirebaseUserWrapper) AbsObjectWrapper.ConvertToWrapper(new FirebaseUIWrapper.FirebaseUserWrapper(), auth.getCurrentUser());
		} else {
			return null;
		}
  	//return (FirebaseUIWrapper.FirebaseUserWrapper)AbsObjectWrapper.ConvertToWrapper(new FirebaseUIWrapper.FirebaseUserWrapper(), this.auth.getCurrentUser());
  }

  
//  @SuppressWarnings("unchecked")
//	public void GetUserTokenId(final BA ba, final FirebaseUIWrapper.FirebaseUserWrapper User, boolean ForceRefresh) {
//    User.getObject().getToken(ForceRefresh).addOnSuccessListener(new OnSuccessListener() {
//			@Override
//			public void onSuccess(Object arg0) {
//				// TODO Auto-generated method stub
//				GetTokenResult result = (GetTokenResult)arg0;
//        ba.raiseEventFromDifferentThread(FirebaseUIWrapper.this, (Object)null, 0, FirebaseUIWrapper.this.eventName + "_tokenavailable", false, new Object[]{User, Boolean.valueOf(true), result.getToken()});
//			}
//
//    }).addOnFailureListener(new OnFailureListener() {
//       @Override
//			public void onFailure(Exception arg0) {
//          ba.raiseEventFromDifferentThread(FirebaseUIWrapper.this, (Object)null, 0, FirebaseUIWrapper.this.eventName + "_tokenavailable", false, new Object[]{User, Boolean.valueOf(false), ""});
//       }
//    });
// }

  private int GetResourceId(String Type, String Name) {
    return BA.applicationContext.getResources().getIdentifier(Name, Type, BA.packageName);
 }

  @ShortName("FirebaseUser")
  public static class FirebaseUserWrapper extends AbsObjectWrapper<FirebaseUser> {
  	public String getEmail() {
  		return this.getObject().getEmail();
  	}
  	public boolean isAnonymous() {
  		return this.getObject().isAnonymous();
  	}
  	public boolean isEmailVerified() {
  		return this.getObject().isEmailVerified();
  	}

     public String getDisplayName() {
        return this.getObject().getDisplayName();
     }

     public String getUid() {
        return this.getObject().getUid();
     }

     public String getPhotoUrl() {
        return this.getObject().getPhotoUrl() == null?"":this.getObject().getPhotoUrl().toString();
     }
  }

}
